@@ -1,9 +1,10 @@
-from abc import abstractmethod
 from typing import Union
 
+from treepath.path.exceptions.set_error import SetError
 from treepath.path.traverser.list_match import ListMatch
 from treepath.path.traverser.traverser_match import TraverserMatch
-from treepath.path.util.function import enumerate_slice
+from treepath.path.typing.json_types import JsonTypes
+from treepath.path.utils.function import enumerate_slice
 from treepath.path.vertex.vertex import Vertex
 
 
@@ -17,9 +18,8 @@ class _ListVertex(Vertex):
     def path_segment(self):
         return f"[{self.name}]"
 
-    @abstractmethod
     def match(self, parent_match: TraverserMatch, traverser, vertex_index: int) -> Union[TraverserMatch, None]:
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: no cover
 
 
 class ListIndexVertex(_ListVertex):
@@ -48,6 +48,36 @@ class ListIndexVertex(_ListVertex):
         except IndexError:
             pass
 
+    @property
+    def default_value_for_set(self) -> Union[dict, list]:
+        return list()
+
+    def set(self, parent_match: TraverserMatch, value: JsonTypes) -> TraverserMatch:
+        data = parent_match.data
+        if isinstance(data, list):
+            try:
+                data[self.name] = value
+            except IndexError:
+                if len(data) != self.name:
+                    raise SetError(
+                        self.parent,
+                        f"The path {self} index is out of range.  "
+                        f"The index must be in the range 0 >= index <={len(data)}.  To append the index must the list "
+                        f"current length.",
+                        self.path_segment
+                    )
+                data.append(value)
+            return self.match(parent_match, None, parent_match.vertex_index + 1)
+        else:
+            self.raise_invalid_set(data, value)
+
+    def pop(self, match: TraverserMatch) -> JsonTypes:
+        data = match.parent.data
+        if isinstance(data, list):
+            return data.pop(self.name)
+        else:
+            self.raise_invalid_pop(data)
+
 
 class ListSliceVertex(_ListVertex):
     __slots__ = '_slice'
